package net.stawrul;

import org.springframework.beans.factory.ObjectProvider;
import org.springframework.boot.autoconfigure.security.oauth2.authserver.AuthorizationServerProperties;
import org.springframework.boot.autoconfigure.security.oauth2.authserver.OAuth2AuthorizationServerConfiguration;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.oauth2.common.DefaultOAuth2AccessToken;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableAuthorizationServer;
import org.springframework.security.oauth2.config.annotation.web.configurers.AuthorizationServerEndpointsConfigurer;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.client.BaseClientDetails;
import org.springframework.security.oauth2.provider.token.AccessTokenConverter;
import org.springframework.security.oauth2.provider.token.TokenEnhancer;
import org.springframework.security.oauth2.provider.token.TokenStore;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static java.util.stream.Collectors.toList;

@Configuration
@EnableAuthorizationServer
public class AuthorizationServerConfig extends OAuth2AuthorizationServerConfiguration {

    public AuthorizationServerConfig(BaseClientDetails details, AuthenticationManager authManager,
                                     ObjectProvider<TokenStore> store, ObjectProvider<AccessTokenConverter> converter,
                                     AuthorizationServerProperties properties) {
        super(details, authManager, store, converter, properties);
    }

    @Override
    public void configure(AuthorizationServerEndpointsConfigurer endpoints) throws Exception {
        super.configure(endpoints);

        endpoints.reuseRefreshTokens(false);
        endpoints.tokenEnhancer(new AuthoritiesTokenEnhancer());
    }

    static class AuthoritiesTokenEnhancer implements TokenEnhancer {
        @Override
        public OAuth2AccessToken enhance(OAuth2AccessToken accessToken, OAuth2Authentication authentication) {
            DefaultOAuth2AccessToken token = new DefaultOAuth2AccessToken(accessToken);
            addAuthorities(token, authentication);
            return token;
        }

        private void addAuthorities(DefaultOAuth2AccessToken token, OAuth2Authentication authentication) {
            Map<String, Object> addons = new HashMap<>(token.getAdditionalInformation());
            List<String> roles = authentication.getAuthorities().stream()
                    .map(GrantedAuthority::getAuthority).collect(toList());
            addons.put("authorities", roles);
            addons.put("username", authentication.getName());
            token.setAdditionalInformation(addons);
        }
    }
}
