package net.stawrul.controllers;

import net.stawrul.model.Book;
import net.stawrul.model.User;
import net.stawrul.services.BooksService;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.security.RolesAllowed;
import java.net.URI;
import java.util.List;
import java.util.UUID;

import static java.util.stream.Collectors.toList;
import static java.util.stream.StreamSupport.stream;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;
import static org.springframework.http.HttpStatus.CONFLICT;
import static org.springframework.http.ResponseEntity.notFound;
import static org.springframework.http.ResponseEntity.ok;

@RestController
@RequestMapping("/books")
public class BooksController {

    final BooksService booksService;

    public BooksController(BooksService booksService) {
        this.booksService = booksService;
    }

    @GetMapping
    public List<Book> listBooks(@RequestParam(name = "limit", required = false) Integer limit) {
        limit = limit != null ? limit : Integer.MAX_VALUE;
        return stream(booksService.findAllBooks(limit).spliterator(), false).collect(toList());
    }

    @PostMapping
    public ResponseEntity<Void> addBook(@RequestBody Book book) {

        if (booksService.findBook(book.getId()) == null) {
            booksService.save(book);

            URI location = linkTo(methodOn(this.getClass()).getBook(book.getId())).toUri();
            return ResponseEntity.created(location).build();

        } else {
            return ResponseEntity.status(CONFLICT).build();
        }
    }

    @GetMapping("/{id}")
    public ResponseEntity<Book> getBook(@PathVariable UUID id) {
        Book book = booksService.findBook(id);
        return book != null ? ok(book) : notFound().build();
    }

    @RolesAllowed(User.Roles.ADMIN)
    @PutMapping("/{id}")
    public ResponseEntity<Void> updateBook(@PathVariable UUID id, @RequestBody Book book) {

        if (!id.equals(book.getId()) || booksService.findBook(id) == null) {
            return notFound().build();
        }

        booksService.save(book);
        return ok().build();
    }



}
