package net.stawrul.controllers;

import net.stawrul.model.Order;
import net.stawrul.services.OrdersService;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.net.URI;
import java.util.UUID;

import static java.util.Objects.isNull;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;
import static org.springframework.http.ResponseEntity.*;

@RestController
public class ShopController {

    final OrdersService ordersService;

    public ShopController(OrdersService ordersService) {
        this.ordersService = ordersService;
    }


    @GetMapping("/orders")
    public Iterable<Order> listOrders() {
        return ordersService.findAll();
    }

    @GetMapping("/orders/{id}")
    public ResponseEntity<Order> getOrder(@PathVariable UUID id) {
        Order order = ordersService.find(id);
        return isNull(order) ? notFound().build() : ok(order);
    }

    @PostMapping("/orders")
    public ResponseEntity<Void> addOrder(@RequestBody Order order) {
        ordersService.placeOrder(order);
        URI location = linkTo(methodOn(this.getClass()).getOrder(order.getId())).toUri();
        return created(location).build();
    }
}
