package net.stawrul.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.hibernate.validator.constraints.NotBlank;

import javax.persistence.*;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import static java.lang.String.format;

@EqualsAndHashCode(of = "id", callSuper = false)
@NoArgsConstructor
@Entity
@NamedQueries({
        @NamedQuery(name = Author.Queries.COUNT_BOOKS_PER_AUTHOR, query = "SELECT a, COUNT(b) FROM Author a LEFT JOIN a.books b GROUP BY a"),
})
public class Author implements Serializable {

    public static class Queries {
        public static final String COUNT_BOOKS_PER_AUTHOR = "Author.countBooksPerAuthor";
    }

    @Getter
    @Id
    UUID id = UUID.randomUUID();

    @Getter
    @Setter
    @NotBlank
    private String name;

    @Getter
    @Setter
    @NotBlank
    private String surname;

    @Getter
    @Setter
    @ManyToMany(mappedBy = "authors")
    @JsonIgnore
    private List<Book> books = new ArrayList<>();

    public Author(String name, String surname) {
        this.name = name;
        this.surname = surname;
    }

    @JsonIgnore
    public String getFullName() {
        return format("%s %s", this.getName(), this.getSurname());
    }
}
