package net.stawrul.services;

import net.stawrul.model.Book;
import net.stawrul.model.Order;
import net.stawrul.model.User;
import net.stawrul.services.exceptions.OutOfStockException;
import net.stawrul.services.repositories.BooksRepository;
import net.stawrul.services.repositories.OrdersRepository;
import net.stawrul.services.repositories.UsersRepository;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.UUID;

@Service
public class OrdersService {

    OrdersRepository ordersRepository;
    BooksRepository booksRepository;
    UsersRepository usersRepository;

    public OrdersService(OrdersRepository ordersRepository,
                         BooksRepository booksRepository,
                         UsersRepository usersRepository) {
        this.ordersRepository = ordersRepository;
        this.booksRepository = booksRepository;
        this.usersRepository = usersRepository;
    }

    public Iterable<Order> findAll() {
        return ordersRepository.findAllByUser(getCurrentUser());
    }

    public Order find(UUID id) {
        return ordersRepository.findOne(id);
    }
    
    @Transactional
    public void placeOrder(Order order) {
        User user = getCurrentUser();
        order.setUser(user);

        int subtotal = 0;
        
        for (Book bookStub : order.getBooks()) {
            Book book = booksRepository.findOne(bookStub.getId());
            subtotal += book.getPrice();

            if (book.getAmount() < 1) {
                throw new OutOfStockException();
            } else {
                int newAmount = book.getAmount() - 1;
                book.setAmount(newAmount);
            }
        }

        order.setSubtotal(subtotal);
        ordersRepository.save(order);
    }

    private User getCurrentUser() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        String login = authentication.getName();
        return usersRepository.findOne(login);
    }
}
