package net.stawrul.services.repositories;

import net.stawrul.model.Author;
import net.stawrul.model.Book;
import net.stawrul.model.User;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;

import static java.util.Arrays.asList;

@Service
public class InitialFixture {

    public InitialFixture(BooksRepository booksRepository,
                          UsersRepository usersRepository,
                          AuthorsRepository authorsRepository,
                          PasswordEncoder encoder) throws ParseException {

        if (authorsRepository.count() == 0) {
            List<Author> authors = asList(
                    new Author("Orson Scott", "Card"),
                    new Author("Aaron", "Johnston"),
                    new Author("Maciej", "Guzek"),
                    new Author("Maja", "Kossakowska"),
                    new Author("Neil", "Gaiman")
            );
            authorsRepository.save(authors);

            DateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            List<Book> books = asList(
                    new Book("Gra Endera", format.parse("1988-02-15"), asList(authors.get(0)), 0, 3599),
                    new Book("Ksenocyd", format.parse("1992-02-15"), asList(authors.get(0)), 6, 3599),
                    new Book("Mówca umarłych", format.parse("1994-02-15"), asList(authors.get(0)), 5, 3599),
                    new Book("Dzieci umysłu", format.parse("1995-02-15"), asList(authors.get(0)), 12, 3599),
                    new Book("Trzeci Świat", format.parse("2009-10-01"), asList(authors.get(2)), 9, 4490),
                    new Book("Ruda Sfora", format.parse("2011-01-14"), asList(authors.get(3)), 6, 3499),
                    new Book("W Przededniu", format.parse("2013-03-19"), asList(authors.get(0), authors.get(1)), 14, 2999)
            );
            booksRepository.save(books);
        }

        if (usersRepository.count() == 0) {
            usersRepository.save(new User("user1", encoder.encode("password1"), asList(User.Roles.USER)));
            usersRepository.save(new User("admin1", encoder.encode("drowssap1"), asList(User.Roles.ADMIN)));
        }
    }
}
