import {NgModule} from '@angular/core';
import {RouterModule, Routes} from '@angular/router';

import {SignInComponent} from './components/sign-in/sign-in.component';
import {BooksComponent} from './components/books/books.component';
import {AuthorsComponent} from './components/authors/authors.component';
import {BookDetailsComponent} from './components/book-details/book-details.component';
import {DashboardComponent} from './components/dashboard/dashboard.component';
import {CartComponent} from './components/cart/cart.component';
import {OrdersComponent} from './components/orders/orders.component';
import {BookEditComponent} from './components/book-edit/book-edit.component';
import {AuthenticatedGuard} from './guards/authenticated-guard.service';
import {AdminGuard} from './guards/admin-guard.service';

const routes: Routes = [
  {path: '', redirectTo: '/sign-in', pathMatch: 'full'},
  {path: 'sign-in', component: SignInComponent},
  {path: 'dashboard', component: DashboardComponent, canActivate: [AuthenticatedGuard]},
  {path: 'books', component: BooksComponent, canActivate: [AuthenticatedGuard]},
  {path: 'books/:id', component: BookDetailsComponent, canActivate: [AuthenticatedGuard]},
  {path: 'books/:id/edit', component: BookEditComponent, canActivate: [AuthenticatedGuard, AdminGuard]},
  {path: 'authors', component: AuthorsComponent, canActivate: [AuthenticatedGuard]},
  {path: 'cart', component: CartComponent, canActivate: [AuthenticatedGuard]},
  {path: 'orders', component: OrdersComponent, canActivate: [AuthenticatedGuard]},
];

@NgModule({
  imports: [RouterModule.forRoot(routes)],
  exports: [RouterModule]
})
export class AppRoutingModule {
}
