import {BrowserModule} from '@angular/platform-browser';
import {NgModule} from '@angular/core';
import {HTTP_INTERCEPTORS, HttpClientModule} from '@angular/common/http';
import {FormsModule} from '@angular/forms';

import {CalendarModule} from 'primeng/primeng';

import {AppComponent} from './components/app.component';
import {NavbarComponent} from './components/navbar/navbar.component';
import {SignInComponent} from './components/sign-in/sign-in.component';
import {AppRoutingModule} from './app-routing.module';
import {BooksComponent} from './components/books/books.component';
import {BookDetailsComponent} from './components/book-details/book-details.component';
import {AuthorsComponent} from './components/authors/authors.component';
import {BooksService} from './services/books.service';
import {AuthService} from './services/auth.service';
import {API_BASE_URL, API_BASE_URL_VALUE} from './app.config';
import {DashboardComponent} from './components/dashboard/dashboard.component';
import {CartService} from './services/cart.service';
import {CartComponent} from './components/cart/cart.component';
import {OrdersService} from './services/orders.service';
import {OrdersComponent} from './components/orders/orders.component';
import {BookEditComponent} from './components/book-edit/book-edit.component';
import {AuthInterceptor} from './interceptors/auth-interceptor';
import {IdentityService} from './services/identity.service';
import {AuthenticatedGuard} from './guards/authenticated-guard.service';
import {AdminGuard} from './guards/admin-guard.service';


@NgModule({
  imports: [
    BrowserModule,
    AppRoutingModule,
    HttpClientModule,
    FormsModule,
    CalendarModule
  ],
  declarations: [
    AppComponent,
    NavbarComponent,
    SignInComponent,
    BooksComponent,
    BookDetailsComponent,
    AuthorsComponent,
    DashboardComponent,
    CartComponent,
    OrdersComponent,
    BookEditComponent,
  ],
  providers: [
    {
      provide: API_BASE_URL,
      useValue: API_BASE_URL_VALUE
    },
    {
      provide: HTTP_INTERCEPTORS,
      useClass: AuthInterceptor,
      multi: true
    },
    BooksService,
    AuthService,
    CartService,
    OrdersService,
    IdentityService,

    AuthenticatedGuard,
    AdminGuard
  ],
  bootstrap: [AppComponent]
})
export class AppModule {
}
