import {Component, OnInit} from '@angular/core';
import {Book} from '../../model/book';
import {BooksService} from '../../services/books.service';
import {ActivatedRoute, Router} from '@angular/router';
import {CartService} from '../../services/cart.service';
import {IdentityService} from '../../services/identity.service';

@Component({
  selector: 'app-book-details',
  templateUrl: './book-details.component.html',
  styleUrls: ['./book-details.component.css']
})
export class BookDetailsComponent implements OnInit {
  book: Book;

  constructor(private bookService: BooksService,
              private cartService: CartService,
              private router: Router,
              private route: ActivatedRoute,
              private identityService: IdentityService) {
  }

  ngOnInit() {
    const id = this.route.snapshot.paramMap.get('id');
    this.bookService.getBook(id).subscribe(book => this.book = book);
  }

  addToCart(): void {
    this.cartService.addToCart(this.book);
    this.router.navigate(['cart']);
  }

  canEdit(): boolean {
    return this.identityService.isUserInRole('ROLE_ADMIN');
  }
}
