import {Injectable} from '@angular/core';
import {HttpErrorResponse, HttpEvent, HttpHandler, HttpInterceptor, HttpParams, HttpRequest, HttpResponse} from '@angular/common/http';
import {Observable} from 'rxjs/Observable';
import 'rxjs/add/operator/do';

import {UserInfo} from '../model/user-info';
import {USER_INFO_KEY} from '../app.config';
import {Router} from '@angular/router';
import {IdentityService} from '../services/identity.service';

@Injectable()
export class AuthInterceptor implements HttpInterceptor {

  constructor(private router: Router,
              private identityService: IdentityService) {
  }

  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {

    if (this.getAccessToken() != null) {
      request = request.clone({
        setHeaders: {
          Authorization: `Bearer ${this.getAccessToken()}`
        }
      });
    }

    return next.handle(request).do(() => {
      },
      (err: any) => {
        if (err instanceof HttpErrorResponse) {
          if (err.status === 401) {
            this.identityService.signOut();
            this.router.navigate(['sign-in'], {queryParams: {tokenExpired: true}});
          }
        }
      });
  }

  public getAccessToken() {
    const userInfo: UserInfo = JSON.parse(localStorage.getItem(USER_INFO_KEY));
    return userInfo != null ? userInfo.accessToken : null;
  }
}
