import {Inject, Injectable} from '@angular/core';
import {HttpClient, HttpHeaders} from '@angular/common/http';
import {Observable} from 'rxjs/Observable';
import {API_BASE_URL} from '../app.config';
import {UserInfo} from '../model/user-info';
import {IdentityService} from './identity.service';

@Injectable()
export class AuthService {
  private readonly client_id = 'angular-client';
  private readonly client_secret = 'angular-secret';

  private readonly tokenUrl = '/oauth/token';

  constructor(@Inject(API_BASE_URL) private baseUrl: string,
              private http: HttpClient,
              private identityService: IdentityService) {
  }

  public signIn(login: string, password: string): Observable<boolean> {
    const options = {
      headers: new HttpHeaders({
        'Authorization': 'Basic ' + btoa(this.client_id + ':' + this.client_secret),
        'Content-Type': 'application/x-www-form-urlencoded'
      }),
    };

    const body = new URLSearchParams();
    body.set('grant_type', 'password');
    body.set('username', login);
    body.set('password', password);
    body.set('client_secret', this.client_id);

    return Observable.create(observer => {
      this.http.post(this.baseUrl + this.tokenUrl, body.toString(), options)
        .subscribe((res: any) => {
          this.handleAuthSuccess(login, res, observer);
        }, () => {
          this.handleAuthError(observer);
        });
    });
  }


  private handleAuthError(observer) {
    observer.next(false);
  }

  private handleAuthSuccess(login: string, res: any, observer) {
    const userInfo: UserInfo = {
      login: login,
      accessToken: res.access_token,
      refreshToken: res.refresh_token,
      authorities: res.authorities
    };

    this.identityService.storeUserInfo(userInfo);

    observer.next(true);
  }
}
