import {Inject, Injectable} from '@angular/core';
import {HttpClient, HttpParams} from '@angular/common/http';
import {Book} from '../model/book';
import {Observable} from 'rxjs/Observable';
import 'rxjs/add/operator/map';
import {API_BASE_URL} from '../app.config';
import {AuthService} from './auth.service';
import {Author} from '../model/author';

@Injectable()
export class BooksService {
  private booksUrl = '/books';
  private authorsUrl = '/authors';

  constructor(@Inject(API_BASE_URL) private baseUrl: string,
              private authService: AuthService,
              private http: HttpClient) {
  }

  public getBooks(limit?: number): Observable<Book[]> {
    let params = new HttpParams();

    if (limit) {
      params = params.set('limit', limit.toString());
    }

    return this.http.get<Book[]>(this.baseUrl + this.booksUrl, {params: params});
  }

  public getBook(id: string): Observable<Book> {
    return this.http.get<Book>(this.baseUrl + this.booksUrl + '/' + id);
  }

  public getAuthors(): Observable<Author[]> {
    return this.http.get<Author[]>(this.baseUrl + this.authorsUrl);
  }

  public save(book: Book): Observable<Object> {
    return this.http.put(this.baseUrl + this.booksUrl + '/' + book.id, book);
  }
}
