import {Inject, Injectable} from '@angular/core';
import {API_BASE_URL} from '../app.config';
import {HttpClient} from '@angular/common/http';
import {Observable} from 'rxjs/Observable';
import {Order} from '../model/order';
import {AuthService} from './auth.service';
import {Book} from '../model/book';
import {UserInfo} from '../model/user-info';
import {Cart} from '../model/cart';

@Injectable()
export class OrdersService {
  private ordersUrl = '/orders';

  constructor(@Inject(API_BASE_URL) private baseUrl: string,
              private authService: AuthService,
              private http: HttpClient) {
  }

  getOrders(): Observable<Order[]> {
    return this.http.get<Order[]>(this.baseUrl + this.ordersUrl);
  }

  placeOrder(cart: Cart): Observable<boolean> {
    const order = new Order();
    order.books = cart.products;

    return Observable.create(observer => {
      this.http.post(this.baseUrl + this.ordersUrl, order).subscribe(
        () => {
          observer.next(true);
        },
        () => {
          observer.next(false);
        });
    });
  }

}
